"""

    Base class for local file proxy implementations

"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Zope imports
from AccessControl import ClassSecurityInfo
from Globals import InitializeClass
from DateTime.DateTime import DateTime

# Plone imports
from Products.CMFCore.FSObject import FSObject

class BaseProxy(FSObject):
    """ Proxy base class which fakes file objects as Plone objects for Plone

    TODO: Inherited from FSObject. At least getId doesn't work
    if not inherited. See which attributes would have to be 
    implemented ourselves and rip off need for FSObject.
    """ 

    security = ClassSecurityInfo()
    meta_type = 'PloneLocalFolderCatalogProxy'

    security.declarePublic('ModificationDate')
    def ModificationDate(self):
        """ None """
        try:
            return DateTime(os.stat(self._filepath)[8])
        except:
            return DateTime()

    def Description(self):
        """ Description for this file """
        return 'File system file'
    
    def txng_get(self, attr):
        """ TextIndexNG support method that returns the source
        (file body contents), mime type and encoding type of the file """
        
        if attr[0].strip() in ('PrincipiaSearchSource', 'SearchableText'):
           fp = self.url
           file = open(fp, 'rb')
           try: source = file.read()
           finally: file.close()
           #msg = 'txng_get() returned: source=%s;mimetype=%s; encoding=%s' % (source, self.mime_type, self.encoding)
           #zLOG.LOG('PloneLocalFolderNG',zLOG.INFO, msg)
           return source, self.mime_type, self.encoding

        elif attr[0].strip() == 'id':
           source = self.id
           return source, self.mime_type, self.encoding

        else:
           #zLOG.LOG('PloneLocalFolderNG',zLOG.INFO, 'txng_get() returned None')
           return None
       
    security.declarePublic('getIcon')
    def getIcon(self, arg=None):
        """ icon """
        return self.icon_path       
       
    security.declarePublic('absolute_url')
    def absolute_url(self,relative=0):
        """ return url """
        print "!!! absolute_url"
        return self.url

    security.declarePublic('title_or_id')
    def title_or_id(self):
        """ return title or id """
        return self.id
        
    def Title(self):
        """ Method used by portlet_navtree_macro.pt """
        return self.id
    
    security.declarePublic('getTitle')
    def getTitle(self):
        return self.title

       
InitializeClass(BaseProxy)       