"""
    Cataloging object for local folder files
    
    Copyright 2006 Mikko Ohtamaa

"""

__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os

# Zope imports
from Acquisition import Acquired, aq_inner, aq_parent, aq_base
from AccessControl import ClassSecurityInfo
from DateTime.DateTime import DateTime
from Globals import InitializeClass
from OFS.SimpleItem import SimpleItem
import zLOG

# Plone imports
from Products.CMFCore.FSObject import FSObject
from Products.Archetypes.public import BaseContent, BaseSchema, Schema

# Local imports
from BaseProxy import BaseProxy

class CatalogProxy(BaseProxy):
    """ Cataloging object for local folder files

    CatalogProxy proxies information between PloneLocalFolderNG
    mounted files and portal_catalog.catalog_file
    
    Responsibilities:
        - Give path for cataloging (path, set by utils.catalogFS)
        - Give object type for cataloging (portal_type)
        - Give misc. info for cataloging
    """

    schema = BaseSchema + Schema([])
    security = ClassSecurityInfo()
    
    # Navtree generation doesn't work unless portal_type is given for portal_catalog
    # during catalogTool.catalog_object and portal_type is available in portal_types tool
    portal_type = "PloneLocalFolderCatalogProxy"
    language  = 'neutral'
    
    def __init__(self, **kwargs):
        """ Construct CatalogProxy object
                  
        Variables __init__ sets from kwargs are used by portal_catalog
        when indexing this object.
        
        @param path Portal path for this file
        @param url Absolute url for the object
        @param iconpath Icon for this file
        @param mimetype mimetype for this file'
        @param view_roles sequence of role names allowed to view this file
        """ 
        
        # id is filename 
        self.id =  kwargs["id"]
        
        if self.id == None or len(self.id) == 0:
            raise AttributeError, "Invalid id"
        
        # self.url = url
        self.path = kwargs["path"]
        self.icon_path = kwargs["icon_path"]
        self.mime_type = kwargs["mime_type"]
        self.encoding = kwargs["encoding"]
        self.effective = kwargs["effective"]
        self.expires = kwargs["expires"]
        
        # TODO: WTF is this variable hack
        self._View_Permission = kwargs["view_roles"]
        
        # Affects PloneTool.createNavTree show_children
        # Makes folder files appear in sitemap
        self.isPrincipiaFolderish= kwargs["is_folderish"]
        
        # makes file content searchable
        self.catalog_text = kwargs["catalog_text"]

    security.declarePublic('Language')
    def Language(self):
        return self.language
        
    def getPath(self):
        raise "Should not happen"

InitializeClass(CatalogProxy)
