"""
    Proxy for listFolderContent() items
    
    Copyright 2006 Andreas Jung, Mike Garnsey, Mikko Ohtamaa

"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os

# Zope imports
import zLOG
from Globals import InitializeClass, Persistent
from AccessControl import ClassSecurityInfo
from OFS.SimpleItem import SimpleItem

# Local imports
from BaseProxy import BaseProxy

class FileProxy(BaseProxy):
    """A fake File proxy class """

    security = ClassSecurityInfo()

    language  = 'neutral'
    checksum = ''
    revision = ''
    portal_type = 'Document'
    
    security.declarePublic('getChecksum')
    def getChecksum(self):
        """ return checksum"""
        if self.checksum:
            return self.checksum
        else:
            return ''

    security.declarePublic('getComment')
    def getComment(self):
        """ return comment"""
        return self.comment

    security.declarePublic('getRevision')
    def getRevision(self):
        """ return revision"""
        return self.revision

    security.declarePublic('get_size')
    def get_size(self):
        """ None """
        if os.path.isdir(self._filepath):
           return ''
        try:
           size = os.stat(self._filepath)[6]
           return size
        except: return ''

    security.declarePublic('Language')
    def Language(self):
        return self.language

    def setAbsoluteURL(self, url):
        self.url = url

    def setChecksum(self, checksum):
        self.checksum = checksum

    def setComment(self, comment):
        self.comment = comment

    def setIconPath(self, icon_path):
        self.icon_path = icon_path

    def setLanguage(self, language='neutral'):
        self.language = language

    def setMimeType(self, mt):
        self.mime_type = mt

    def setRevision(self, revision):
        self.revision = revision

    def setTitle(self, title):
        self.title = title

    security.declarePublic('absolute_url')
    def absolute_url(self,relative=0):
        """ return url """
        return self.url
    
    security.declarePublic('getTypeInfo')
    def getTypeInfo(self):
        """ return type info """
        TI = TypeInfo(self.id, self.mime_type)
        TI.setMimeType(self.mime_type)
        return TI    

    def _readFile(self, *args, **kw):
        """ read the file """

InitializeClass(FileProxy)

class TypeInfo(SimpleItem):
    """ fake TypeInfo class to make CMF happy """

    __allow_acces_to_unprotected_subobjects__ = 1

    security = ClassSecurityInfo()
    security.declareObjectPublic()
    security.setDefaultAccess('allow')

    immediate_view = 'view'

    def setMimeType(self, mt):
        self.mime_type = mt

    security.declarePublic('getActionById')
    def getActionById(self, id, default=None):
        """ None """
        return None

    security.declarePublic('Title')
    def Title(self):
        """ None """
        return self.mime_type

InitializeClass(TypeInfo)