"""
    Utility class for dealing with PloneLocalFolderNG
    
    Copyright 2006 Mike Garnsey, Red Innovation

"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Zope imports
from DateTime.DateTime import DateTime
from AccessControl import ClassSecurityInfo
from AccessControl.Permission import Permission
from OFS.SimpleItem import SimpleItem 
from Globals import InitializeClass 

# Plone imports
from Products.CMFCore.utils import UniqueObject 
from Products.CMFCore.utils import getToolByName

# Local imports
from LocalFolderFilterType import LocalFolderFilterType

class PloneLocalFolderNGTool (UniqueObject, SimpleItem): 
  """ PloneLocalFolderNGTool """ 
  id = 'plone_local_folder_tool' 
  meta_type= 'PloneLocalFolderNG Tool' 
  plone_tool = 1 
  
  localFolderFilters = []
  
  usedMimeTypes = []
  
  security = ClassSecurityInfo()
  
  def __init__(self):
      """ populate a default localFolderFilter to this tool """

      if len(self.localFolderFilters) == 0:
          self.localFolderFilters.append(LocalFolderFilterType())

  def addLocalFolderFilter(self, **kwargs):
      """ add a localFolderFilter to this tool """
      id = kwargs['id']
      existingFilterIds = self.getLocalFolderFilterIdList()
      if id and not id in existingFilterIds:
          self.localFolderFilters.append(LocalFolderFilterType(kwargs))
          return 1
      else:
          return 0

  def updateLocalFolderFilter(self, **kwargs):
      """ add a localFolderFilter to this tool """
      
      filterToUpdate = None

      id = kwargs.get('id', '')

      for filter in self.localFolderFilters:
          if id == filter.id:
              filterToUpdate = filter
              break
      if filterToUpdate:
          filterToUpdate.set(kwargs)
          return 1
      else:
          return 0

  def dumpLocalFolderFilter(self, filterId):
      """ return the key-value attribute info for a localFolderFilter """
      
      filterToDump = None
      resultList = []

      for filter in self.localFolderFilters:
          if filterId == filter.id:
              filterToDump = filter
              break
      
      if filterToDump:
          resultList.append('hidden_file_prefixes:'+filterToDump.hidden_file_prefixes)
          resultList.append('hidden_file_suffixes:'+filterToDump.hidden_file_suffixes)
          resultList.append('hidden_file_names:'+filterToDump.hidden_file_names)
          resultList.append('hidden_folder_prefixes:'+filterToDump.hidden_folder_prefixes)
          resultList.append('hidden_folder_suffixes:'+filterToDump.hidden_folder_suffixes)
          resultList.append('hidden_folder_names:'+filterToDump.hidden_folder_names)
      
      return resultList

  def getLocalFolderFilterIdList(self):
      """ return the list of id's for the current set of localFolderFilters """
      filterNames = []
      for filter in self.localFolderFilters:
          filterNames.append(filter.id)
      return filterNames     
  
  security.declarePublic('normalizeCSSStyleName')
  def normalizeCSSStyleName(self, filename):
      """ Normalizes string so that in can be used in CSS style
      """
      plone_utils = getToolByName(self, 'plone_utils')
      normalized = plone_utils.normalizeString(filename)
      normalized = normalized.replace(".", "-")
      normalized = normalized.replace("_", "-")
      return normalized
      
  
  security.declarePublic('getTypeIcons')
  def getTypeIcons(self):        
        """ Get list of all mimetype icons used at the site
        
        Get full navtree and extract icon information from there
        """
        
        ct = getToolByName(self, 'portal_catalog')
        plone_utils = getToolByName(self, 'plone_utils')
        portal_url = getToolByName(self, 'portal_url')
                
        query = {}
        query['path'] = {'query': portal_url.getPortalPath(),
                         'depth':9999
                        }
        query['portal_type'] = plone_utils.typesToList()                        
                        
        print "catalog_tool:" + str(ct)
        
        rawresult = ct(**query)

        # Build result dict
        usedIcons = []
        result = []

        for item in rawresult:            
            normalized = self.normalizeCSSStyleName(item.getIcon)
            if not normalized in usedIcons:
                usedIcons.append(normalized)
                result.append(
                              { "filename" : item.getIcon,
                                "normalized_name" : normalized
                               }
                             )
        return result
        
        
        
  

InitializeClass(PloneLocalFolderNGTool) 