# Python imports
import os, os.path
# Zope imports
from Globals import package_home

# Plone imports
from Products.Archetypes.public import process_types, listTypes
from Products.CMFCore import utils
from Products.CMFCore.DirectoryView import registerDirectory
from Products.validation import validation
from validators import ExistingFolderPathValidator,ExternalMD5UtilityValidator

# Local imports
from config import SKINS_DIR, GLOBALS, PROJECTNAME
from config import ADD_CONTENT_PERMISSION

registerDirectory(SKINS_DIR, GLOBALS)


validation.register(ExistingFolderPathValidator('isValidExistingFolderPath'))
validation.register(ExternalMD5UtilityValidator('isValidExternalMD5Utility'))

def initialize(context):
    """
    @parameter context Zope/App/ProductContext instance
    """
    ##Import Types here to register them
    import PloneLocalFolderNG
    import PloneLocalFolderNGTool
    from CatalogProxy import CatalogProxy

    content_types, constructors, ftis = process_types(
        listTypes(PROJECTNAME),
        PROJECTNAME)
            
    # Catalog proxy must be registered, 
    # otherwise createNavTree ignores CatalogProxy entries
    
    proxy_factory_type_information={}
    proxy_factory_type_information.update( {'id':CatalogProxy.portal_type,
                                  'meta_type':CatalogProxy.meta_type,
                                  'factory':None,
                                  'global_allow':True} )
    
    content_types = content_types + (CatalogProxy,)
    ftis = ftis + (proxy_factory_type_information,)
        
    utils.ContentInit(
        PROJECTNAME + ' Content',
        content_types      = content_types,
        permission         = ADD_CONTENT_PERMISSION,
        extra_constructors = constructors,
        fti                = ftis,
        ).initialize(context)
        
    tools = (PloneLocalFolderNGTool.PloneLocalFolderNGTool,)
    utils.ToolInit(PloneLocalFolderNGTool.PloneLocalFolderNGTool.meta_type, tools=tools,
                       icon='tool.gif',
                       ).initialize(context)        


