"""
    Functional test base for PloneLocalFolderNG 
    
    Copyright 2006 Mikko Ohtamaa <miohtama@mail.student.oulu.fi>
    
    Requires:
        - Plone 2.1

"""

__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os, sys
import time
import urllib
from StringIO import StringIO
from HTMLParser import HTMLParser


if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Zope Imports
from Testing import ZopeTestCase

# Plone imports
from Products.Archetypes.atapi import *
from Products.Archetypes.tests.attestcase import default_user
from Products.Archetypes.tests.attestcase import HAS_PLONE21
from Products.Archetypes.tests.atsitetestcase import portal_owner
from Products.Archetypes.tests.atsitetestcase import ATFunctionalSiteTestCase
from Products.Archetypes.tests.utils import DummySessionDataManager
from Products.Archetypes.tests.utils import makeContent
from Products.CMFPlone.tests.PloneTestCase import PloneTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import *

# Local imports
from Products.PloneLocalFolderNG import initialize
from Products.PloneLocalFolderNG.CatalogProxy import CatalogProxy
from Products.PloneLocalFolderNG.tests.test_plfng import SAMPLES_PATH

# Run __init__.py's initialize(context) for products used in this test case
# This registers types and other stuff
ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PloneLocalFolderNG')
ZopeTestCase.installProduct('PortalTransforms')

class PLFNGFunctionalTestCase(ATFunctionalSiteTestCase):
    """ Functional tests super class for PLFNG
 
    """

    def afterSetUp(self):
        """ Do PloneLocalFolderNG navtree specific common test setup
        
        - Setup PloneLocalFolderNG instance into test environment        
        """
        ATFunctionalSiteTestCase.afterSetUp(self)
        
        # Put dummy sdm and dummy SESSION object into REQUEST
        request = self.app.REQUEST
        
        # Set up our fake session data manager
        # TODO: What is the purpose of DummySessionDataManager?
        if self.app._getOb("session_data_manager"):
            # remove previous session_data_manager created in 
            # ZopeTestCase.utils
            self.app._delObject("session_data_manager")
        
        self.app._setObject('session_data_manager', DummySessionDataManager())
        sdm = self.app.session_data_manager
        request.set('SESSION', sdm.getSessionData())
                
        # fail on any logged exception
        self.portal.error_log._ignored_exceptions = ()
                
        # absolite path for files containing sample folder structure
        self.samples_folder = os.path.join(INSTANCE_HOME, *SAMPLES_PATH.split('/'))

        self.qi = self.portal.portal_quickinstaller
        self.qi.installProduct('Archetypes')
        self.at = self.portal.archetype_tool
        self.qi.installProduct('PloneLocalFolderNG')

        # Register PloneLocalFolderNG actions 
        # etc. skins stuff introduced by installer
        # Must be done manually in ATFunctionalSiteTestCase
        self._refreshSkinData()
                
        plone_tool = self.portal.plone_utils        
                
        # We want 401 responses, not redirects to a login page
        if hasattr(self.portal.aq_base, 'cookie_authentication'):
            self.portal._delObject('cookie_authentication')    

        # Setup authentications which can be used
        # when querying stuff via self.publish()
        
        # normal user
        self.basic_auth = '%s:secret' % default_user
        # portal owner
        # TODO: not used anywhere, no examples availables
        # setRoles is used to define roles. Is portal_owner anymore relevant?
        self.owner_auth = '%s:secret' % portal_owner
        
        # Cataloging needs Manager priviledges        
        self.setRoles(['Manager'])
                    
        # under which place our test instances are created
        self.folder_url = self.portal.absolute_url()
        self.folder_path = '/%s' % self.portal.absolute_url(True)
        
        #print "folder_path:" + self.folder_path
            
        # Create PloneLocalFolderNG instance, id=localfolder
        # directly under portal
        response = self.publish(self.folder_path +
           "/invokeFactory?type_name=PloneLocalFolderNG"
            +"&id=localfolder"
            +"&folder=" + urllib.quote(self.samples_folder),
                       
            self.basic_auth)
        
        #print "headers:" + str(response.headers)
        
        # PloneLocalFolderNG's invokeFactory 
        # redirects to base_edit, no OK code given
        # should work though
        # self.assertStatusEqual(response.getStatus(), 200) # OK

        # Check that localfolder was created 
        self.failUnless("localfolder" in self.portal.objectIds())
        
        url = self.portal.localfolder.absolute_url(relative=True) + "/plfng_catalog?action=catalog_now"
        
        # Catalog files so that file paths are added to portal_catalog
        response = self.publish(url, self.basic_auth)         
        self.assertStatusEqual(response.getStatus(), 200) # OK   
        
        self.failUnless("<b>total # files cataloged = </b>" in response.getBody(), "Cataloging sample files failed")
        
    def assertStatusEqual(self, a, b, msg=''):
        """Helper method that uses the error log to output useful debug infos
        """
        now = time.time()
        if a != b:
            entries = self.portal.error_log.getLogEntries()
            if entries:
                msg = entries[0]['tb_text']
            else:
                if not msg:
                    msg = 'no error log msg available'
                    self.failUnlessEqual(a, b)
        self.failUnlessEqual(a, b, msg)
        
    def checkLink(self, html_data, url):
        """ Returns true if <a> tag with target url exists """    
        
        parser = FindLinkParser(url)
        parser.reset()
        parser.feed(html_data)
        
        if not parser.gotMatch():
            print html_data
            self.fail("No link %s found in HTML data" % url)
        

class FindLinkParser(HTMLParser):            
    """ Fink links in HTML code """
    
    def __init__(self, link):
        self.link = link
        self.found = False
    
    def gotMatch(self):
        return self.found
    
    def handle_starttag(self, tag, attrs):
        if tag == "a":
            for attr in attrs:
                if len(attr) >= 2 and attr[0] == "href" and attr[1] == self.link:
                    self.found = True
                      
        
def print_sitemap(sitemap, indent=0):
    """ Output sitemap in a friendly form to stdout 
    
    Recursive
    
    @param indent: recursive increase printing indent 
    """
    print " " * indent + " getURL:" + sitemap["getURL"]
    
    if "path" in sitemap.keys():        
        print " " * indent + " path:" + sitemap["path"]
        
    for c in sitemap["children"]:
        print_sitemap(c, indent+2)
                                
