"""
    Test PloneLocalFolderNG filtering rules
    
    Copyright 2006 Mikko Ohtamaa

"""
__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os, sys
import time
from StringIO import StringIO
import urllib

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Zope Imports
from Testing import ZopeTestCase

# Plone imports
from Products.Archetypes.atapi import *
from Products.Archetypes.tests.attestcase import default_user
from Products.Archetypes.tests.attestcase import HAS_PLONE21
from Products.Archetypes.tests.atsitetestcase import portal_owner
from Products.Archetypes.tests.atsitetestcase import ATFunctionalSiteTestCase
from Products.Archetypes.tests.utils import DummySessionDataManager
from Products.Archetypes.tests.utils import makeContent
from Products.CMFPlone.tests.PloneTestCase import PloneTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import *

# Local imports
from Products.PloneLocalFolderNG import initialize
from Products.PloneLocalFolderNG.CatalogProxy import CatalogProxy
from Products.PloneLocalFolderNG.tests.test_plfng import SAMPLES_PATH
from test_navtree import print_sitemap

# Run __init__.py's initialize(context) for products used in this test case
# This registers types and other stuff
ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PloneLocalFolderNG')
ZopeTestCase.installProduct('PortalTransforms')


class BaseTest(PloneTestCase):
    samples_folder = os.path.join(INSTANCE_HOME,*SAMPLES_PATH.split('/'))
    firstFileInLocalFolder = None
                                   
    def afterSetUp(self):
        """ bootstrap test with a PLFNG """
        PloneTestCase.afterSetUp(self)
        self.loginPortalOwner()
        self.qi = getToolByName(self.portal, 'portal_quickinstaller')
        self.qi.installProduct('Archetypes')
        self.at = getToolByName(self.portal, 'archetype_tool')
        self.qi.installProduct('PloneLocalFolderNG')
        self.at.manage_installType(typeName='PloneLocalFolderNG',
                                   package='PloneLocalFolderNG')


    def test_list_filtered_content(self):
        """ Test filter rules apply on listing one folder """
        
        portal = self.portal
        
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folder.getField('folder').set(folder, self.samples_folder)
        
        # try list without doc
        folder.getField('hidden_file_suffixes').set(folder, "doc")        
        contents = folder.listFolderContents()        
        self.failUnless(len(contents) == 8, "Without .doc 8 files should be listed, files:" + str(contents))
        
        # try list without doc
        folder.getField('hidden_file_suffixes').set(folder, "doc,jpg")        
        contents = folder.listFolderContents()
        self.failUnless(len(contents) == 7, "Without .doc and .jpg 7 files should be listed, files:" + str(contents))
        
    def test_catalog_non_filtered(self):
        """ Test cataloging works when there are no filter rules """
        
        portal = self.portal
        
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folder.getField('folder').set(folder, self.samples_folder)
        
        folder.catalogFiles()
        
        sitemap = portal.plone_utils.createNavTree(self.portal.localfolder)
        
        #print "items:" + str(countSitemapItems(sitemap))
        #print_sitemap(sitemap)
        
        if countSitemapItems(sitemap) != 14:
            print_sitemap(sitemap)
            self.fail("Sitemap item count wrong:%d" % countSitemapItems(sitemap))
        
    def test_catalog_filtered(self):
        """ Test cataloging works when there are filter rules present """
        
        portal = self.portal
        
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folder.getField('folder').set(folder, self.samples_folder)
        
        # try catakig without .doc files
        folder.getField('hidden_file_suffixes').set(folder, "doc")        
        contents = folder.listFolderContents()        
        
        folder.catalogFiles()        
        
        sitemap = portal.plone_utils.createNavTree(self.portal.localfolder)
        
        #print "items:" + str(countSitemapItems(sitemap))
        print_sitemap(sitemap)
        
        if countSitemapItems(sitemap) != 13:
            print_sitemap(sitemap)
            self.fail("Sitemap item count wrong:%d" % countSitemapItems(sitemap))    
        
def countSitemapItems(sitemap):
    """ Count items in sitemap
    
    Recursive
    """
    
    items = 0
            
    if "path" in sitemap.keys():        
        items += 1        
        
    for c in sitemap["children"]:
        items += countSitemapItems(c)
        
    return items
                                
def test_suite():
    import unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BaseTest))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
