import os, sys, shutil, md5

INSTANCE_HOME = os.environ.get('INSTANCE_HOME')

TEST_MD5FILE_CONTENTS = "PLFNG MD5 unit test file"

# MG: on my windows box, I have the system environment variable
# INSTANCE_HOME=c:\plone\data, but if test_plfng.py is called from
# runalltests.py, INSTANCE_HOME = C:\Plone\Zope\lib\python\Testing
# This causes samples_folder path to be incorrect and fails the test.

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Load fixture
from Testing import ZopeTestCase

# Install our product
ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PloneLocalFolderNG')
ZopeTestCase.installProduct('PortalTransforms')

from Products.CMFCore.utils import getToolByName

# MG: makeContent is new to Archetypes as of v1.3, so I added
# it here so that this test will work with the 'out-of-the-box'
# Plone 2.0.5 bundle with Archetypes v1.25-rc4

try:
   from Products.Archetypes.tests.utils import makeContent
except:
   def makeContent( container, portal_type, id='document', **kw ):
       container.invokeFactory( type_name=portal_type, id=id )
       return getattr( container, id )

from Products.CMFPlone.tests import PloneTestCase

from Products.PloneLocalFolderNG.CatalogProxy import CatalogProxy

SAMPLES_PATH = 'Products/PloneLocalFolderNG/tests/SamplesFiles/'

class BaseTest(PloneTestCase.PloneTestCase):
    samples_folder = os.path.join(INSTANCE_HOME,*SAMPLES_PATH.split('/'))
    firstFileInLocalFolder = None
                                   
    def afterSetUp(self):
        """ bootstrap test with a PLFNG """
        PloneTestCase.PloneTestCase.afterSetUp(self)
        self.loginPortalOwner()
        self.qi = getToolByName(self.portal, 'portal_quickinstaller')
        self.qi.installProduct('Archetypes')
        self.at = getToolByName(self.portal, 'archetype_tool')
        self.qi.installProduct('PloneLocalFolderNG')
        self.at.manage_installType(typeName='PloneLocalFolderNG',
                                   package='PloneLocalFolderNG')

    def test_construction(self):
        """ ensure that we can create a PLFNG instance and see files """
        testResult = 0
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folder.getField('folder').set(folder, self.samples_folder)
        
        print "con: %d os: %d" % (len(folder.contentValues()), len(os.listdir(self.samples_folder)))
        print "con:" + str(folder.contentValues()) + " os:" + str(os.listdir(self.samples_folder))
        
        if len(folder.contentValues()) == len(os.listdir(self.samples_folder)):
            testResult = 1
            print "-----"
            print "test_construction() ... ok"
        
        self.failUnless(testResult)
        

    def test_contentAddDirectories(self):
        """ ensure that we can create subfolders"""
        
        testResult = 0 
        
        print
        print "test_contentAddDirectories() ..."
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folder.getField('folder').set(folder, self.samples_folder)
        
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        # 1st make sure that the folder does not exist
        if os.path.exists(folderFSfullPath):
           try:
              shutil.rmtree(folderFSfullPath)
              print "    WARNING: 'testfolder' should not have existed (it was deleted at start of test)"
           except:
              print "    WARNING: 'testfolder' should not have existed (attempt to delete it at start of test failed)"

        if not os.path.exists(folderFSfullPath):
            folder.create_directory("testfolder", None)
            #print "    FS path of new subfolder: %s" % folderFSfullPath
            if os.path.exists(folderFSfullPath):
                testResult = 1
                print "    created subfolder '%s'" % folderFSfullPath
                print "    verified that associated metadata file '%s.metadata' also exists" % os.path.basename(folderFSfullPath)
            else: 
                print "    new subfolder '%s' not found" % folderFSfullPath

        if os.path.exists(folderFSfullPath):
            sectionList = ['GENERAL','GENERAL']
            optionList = ['comment','title']
            newvalueList = ['test folder comment','test folder title']
            folder.setFileMetadata(sectionList, optionList, newvalueList, "testfolder")

       
        self.failUnless(testResult)

    def test_contentAddFile(self):
        """ ensure that we can add a file"""

        testResult = 0
        print
        print "test_contentAddFile() ..."

        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        folder.getField('folder').set(folder, folderFSfullPath)
        
        
        if folder:
           # find the 1st file in the samples folder to use as the file to add 
           sourceFSfullPath = "not set"
           dirList = os.listdir(self.samples_folder)
           for item in dirList:
               itemFullPath = os.path.join(self.samples_folder,item)
               if os.path.isfile(itemFullPath):
                  sourceFSfullPath = itemFullPath
                  break
           
           destPLFNGrelativePath = ''
           destBasename = "testfile1.tst"
           folder.addFile(sourceFSfullPath, destPLFNGrelativePath, destBasename, 0, None)
           #getFileMetadata(self, section="GENERAL", option="comment", rel_dir='')
           
           filePLFNGrelativePathName = destBasename
           sectionList = ['GENERAL','GENERAL']
           optionList = ['comment','title']
           newvalueList = ['test comment','test title']
           
           folder.setFileMetadata(sectionList, optionList, newvalueList, filePLFNGrelativePathName)
           
           destFSfullPath = os.path.join(folderFSfullPath,destBasename)
           destMetadataFSfullPath = destFSfullPath + '.metadata'
           
           if os.path.exists(destFSfullPath) and os.path.exists(destMetadataFSfullPath):
              testResult = 1
              print "    added file: '%s'" % destFSfullPath
              print "    verified that associated metadata file '%s.metadata' also exists" % destBasename
             
        self.failUnless(testResult)



    def test_contentMD5Mechanism(self):
        """ ensure that we can generate MD5 checksums for files"""
        
        testResult = 0
        print
        print "test_contentMD5Mechanism() ..."
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        folder.getField('folder').set(folder, folderFSfullPath)

        # manually create a new file that we test updateMD5Metadatum() with
        testMD5file_FSfullPath = os.path.join(folderFSfullPath,"md5test.tst")
        testMD5fileMetadata_FSfullPath = testMD5file_FSfullPath+'.metadata'
        
        fileHandle = open ( testMD5file_FSfullPath, 'w' ) 
        fileHandle.write ( TEST_MD5FILE_CONTENTS )
        fileHandle.close()
        
        if os.path.exists(testMD5file_FSfullPath):
            print "    manually created file: '%s'" % testMD5file_FSfullPath
            
            m = md5.new()
            m.update(TEST_MD5FILE_CONTENTS)
            python_md5 = m.hexdigest()
            print "    python generated MD5 = %s" % python_md5 
            
            # 1st we need to verify mode='MD5only' of updateMD5Metadatum()
            
            plfng_md5 = folder.updateMD5Metadatum("md5test.tst", 'MD5only', '')
            if plfng_md5 == python_md5:
                print "    verify 'MD5only' behavior of updateMD5Metadatum() ... ok"
                testResult = 1
            else:
                print "    verify 'MD5only' behavior of updateMD5Metadatum() ... failed!"
            
            # then, test mode='testonly' of updateMD5Metadatum() on a file that
            # doesnt have a corresponding .metadata file
            if testResult == 1:

               if os.path.exists(testMD5fileMetadata_FSfullPath):
                   print "        WARNING! found and deleted unexpected .metadata file"
                   os.remove(testMD5fileMetadata_FSfullPath)
               
               result = folder.updateMD5Metadatum("md5test.tst", 'testonly', '')
               
               print "    verify 'testonly' behavior of updateMD5Metadatum():"
               if result == 0:
                   print "        - when md5test.tst.metadata file is missing ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file is missing ... failed!"
            
            # then, test mode='testonly' of updateMD5Metadatum() on a file that
            # has a corresponding .metadata file but is missing MD5 info
            if testResult == 1:

               sectionList = ['GENERAL','GENERAL']
               optionList = ['comment','title']
               newvalueList = ['test folder comment','test folder title']
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               
               result = folder.updateMD5Metadatum("md5test.tst", 'testonly', '')
               
               if result == 0:
                   print "        - when md5test.tst.metadata file does not contain MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file does not contain MD5 info ... failed!"
            
            # then, test mode='testonly' of updateMD5Metadatum() on a file that
            # has incorrect MD5 info in  .metadata file
            if testResult == 1:

               sectionList = ['DIAGNOSTICS',]
               optionList = ['md5',]
               newvalueList = ['invalid md5!',]
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               
               result = folder.updateMD5Metadatum("md5test.tst", 'testonly', '')
               
               if result == 0:
                   print "        - when md5test.tst.metadata file contains incorrect MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file contains incorrect MD5 info ... failed!"

            
            # then, test mode='testonly' of updateMD5Metadatum() on a file that
            # has correct MD5 info in  .metadata file
            if testResult == 1:

               sectionList = ['DIAGNOSTICS',]
               optionList = ['md5',]
               newvalueList = [plfng_md5,]
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               
               result = folder.updateMD5Metadatum("md5test.tst", 'testonly', '')
               
               if result == -1:
                   print "        - when md5test.tst.metadata file contains correct MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file contains correct MD5 info ... failed!"


            # then, we need to verify mode='update' of updateMD5Metadatum() on a file that
            # doesnt have a corresponding .metadata file

            #
            if testResult == 1:
               metadata_md5 = ''
               print "    verify 'update' behavior of updateMD5Metadatum():"
               if os.path.exists(testMD5fileMetadata_FSfullPath):
                   #print "    manually removed .metadata file"
                   os.remove(testMD5fileMetadata_FSfullPath)
               result = folder.updateMD5Metadatum("md5test.tst", 'update', '')
               
               metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               if result==1 and plfng_md5 == python_md5:
                   print "        - when md5test.tst.metadata file is missing ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file is missing ... failed!"
            
            
            # then, we need to verify mode='update' of updateMD5Metadatum() on a file that
            # has a corresponding .metadata file but is missing MD5 info
            
            if testResult == 1:
               metadata_md5 = ''
               
               if os.path.exists(testMD5fileMetadata_FSfullPath):
                   #print "    manually removed .metadata file"
                   os.remove(testMD5fileMetadata_FSfullPath)

               sectionList = ['GENERAL','GENERAL']
               optionList = ['comment','title']
               newvalueList = ['test comment','test title']
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               
               result = folder.updateMD5Metadatum("md5test.tst", 'update', '')
               metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               if result==1 and metadata_md5 == python_md5:
                   print "        - when md5test.tst.metadata file does not contain MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file does not contain MD5 info ... failed!"

            # then, we need to verify mode='update' of updateMD5Metadatum() on a file that
            # has incorrect MD5 info in  .metadata file
            
            if testResult == 1:
               metadata_md5 = ''
               
               if os.path.exists(testMD5fileMetadata_FSfullPath):
                   #print "    manually removed .metadata file"
                   os.remove(testMD5fileMetadata_FSfullPath)

               sectionList = ['DIAGNOSTICS',]
               optionList = ['md5',]
               newvalueList = ['invalid md5!',]
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               #metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               #print "        old metadata md5 = %s" + metadata_md5
               
               result = folder.updateMD5Metadatum("md5test.tst", 'update', '')
               metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               #print "    updated metadata md5 = %s" + metadata_md5
               if result==1 and metadata_md5 == python_md5:
                   print "        - when md5test.tst.metadata file contains invalid MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file contains invalid MD5 info ... failed!"

            # then, we need to verify mode='update' of updateMD5Metadatum() on a file that
            # has correct MD5 info in  .metadata file
            
            if testResult == 1:
               metadata_md5 = ''
               
               if os.path.exists(testMD5fileMetadata_FSfullPath):
                   #print "    manually removed .metadata file"
                   os.remove(testMD5fileMetadata_FSfullPath)

               sectionList = ['DIAGNOSTICS',]
               optionList = ['md5',]
               newvalueList = [python_md5,]
               folder.setFileMetadata(sectionList, optionList, newvalueList, "md5test.tst")
               #metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               #print "        old metadata md5 = %s" + metadata_md5
               
               result = folder.updateMD5Metadatum("md5test.tst", 'update', '')
               metadata_md5 = folder.getFileMetadata("DIAGNOSTICS", "md5", "md5test.tst")
               #print "    updated metadata md5 = %s" + metadata_md5
               if result==-1 and metadata_md5 == python_md5:
                   print "        - when md5test.tst.metadata file contains valid MD5 info ... ok"
               else:
                   testResult = 0
                   print "        - when md5test.tst.metadata file contains valid MD5 info ... failed!"
            #MG: add cases for .metadata not having MD5 and having invalid MD5

    
        self.failUnless(testResult)

    def test_contentOverwriteAddFile(self):
        """ ensure that we can overwrite an existing file"""
        
        testResult = 0
        print
        print "test_contentOverwriteAddFile() ..."
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        folder.getField('folder').set(folder, folderFSfullPath)

        testfile1_FSfullPath = os.path.join(folderFSfullPath,"testfile1.tst")
        testfile2_FSfullPath = os.path.join(folderFSfullPath,"testfile2.tst")
        
        if not os.path.exists(testfile1_FSfullPath):
           print "    WARNING! could not find existing file '%s'" % \
              testfile1_FSfullPath

        else:
           old_testfile1_md5 = \
              folder.updateMD5Metadatum("testfile1.tst", 'MD5only', '')
           
           print "    found existing file '%s' (MD5=%s)" % \
             (os.path.basename(testfile1_FSfullPath), old_testfile1_md5)
             
           # manually create a new file that we can use to add to existing file
           fileHandle = open ( testfile2_FSfullPath, 'w' ) 
           fileHandle.write ( 'PLFNG unit test file #2.' )
           fileHandle.close()

           testfile2_md5 = \
              folder.updateMD5Metadatum("testfile2.tst", 'MD5only', '')
           print "    manually created file: '%s' (MD5=%s)" % \
              ("testfile2.tst",testfile2_md5)


           # use addFile() to try to overwrite the existing file
           sourceFSfullPath = testfile2_FSfullPath
           targetFSfullPath = testfile1_FSfullPath
           targetFilename = os.path.basename(targetFSfullPath)
           destPLFNGrelativePath = ''
           folder.addFile(sourceFSfullPath,destPLFNGrelativePath, 
                          targetFilename, 0, None)
           
           # make sure that the new MD5 for the existing file is correct
           new_testfile1_md5 = \
              folder.updateMD5Metadatum("testfile1.tst",'MD5only','')
           if new_testfile1_md5 == testfile2_md5:
               testResult = 1
               print "    add '%s' as '%s' ... ok" % \
                 ("testfile2.tst","testfile1.tst")

           # either way, print out the new MD5 for the existing file
           print "    new MD5 for '%s' = %s" % \
                (os.path.basename(testfile1_FSfullPath), new_testfile1_md5)
        
        self.failUnless(testResult)

    def xtest_contentPackUnpack(self):
        """ ensure that we can pack and unpack content"""
        
        testResult = 1
        print
        print "test_contentPackUnpack() ..."
        
        # TODO: Tries to create .zip file with infinite size on Windows
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = self.samples_folder
        folder.getField('folder').set(folder, folderFSfullPath)

        filesList = []
        for item in os.listdir(self.samples_folder):
            itemFSFullPathName = os.path.join(self.samples_folder,item)
            if os.path.isfile(itemFSFullPathName):
                filesList.append(itemFSFullPathName)

        for fname in filesList:
            print "    add file: %s" % fname
        folder.packFiles("test.zip", filesList)
        
        
        
        packedFileFSfullPath = "C:\PloneLocalFolderNG.zip"
        unpackDestPLFNGrelativePath = "disk01/130_full"
        folder.unpackFile(unpackDestPLFNGrelativePath, packedFileFSfullPath)

    def test_contentRemoveFile(self):
        """ ensure that we can delete a file"""
        
        testResult = 0
        print
        print "test_contentRemoveFile() ..."
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        folder.getField('folder').set(folder, folderFSfullPath)

        targetFSfullPath = os.path.join(folderFSfullPath,"testfile1.tst")
        targetMetadataFSfullPath = targetFSfullPath + '.metadata'
        
        if not os.path.exists(targetMetadataFSfullPath):
           print "    WARNING: could not find expected metadata file: %s" % targetMetadataFSfullPath
        if not os.path.exists(targetFSfullPath):
           print "    ERROR: could not find file %s to delete" % targetFSfullPath
        else:
           folder.deleteFile(targetFSfullPath, None, None)
           if not os.path.exists(targetFSfullPath):
              if not os.path.exists(targetMetadataFSfullPath):
                 testResult = 1
                 print "    deleted file: '%s'" % os.path.basename(targetFSfullPath)
                 print "    verified that associated metadata file '%s' also deleted" % os.path.basename(targetMetadataFSfullPath)

        self.failUnless(testResult)

    def test_contentRemoveFolder(self):
        """ ensure that we can delete a subfolder"""
        
        testResult = 0
        print
        print "test_contentRemoveFolder() ..."
        
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
        folderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        folder.getField('folder').set(folder, folderFSfullPath)

        targetFolderFSfullPath = os.path.join(self.samples_folder,"testfolder")
        targetFolderMetadataFSfullPath = \
           os.path.join(self.samples_folder,"testfolder.metadata")
        
        if not os.path.exists(targetFolderMetadataFSfullPath):
           print "    WARNING: could not find expected metadata file: %s" % \
              targetFolderMetadataFSfullPath
        
        if not os.path.exists(targetFolderFSfullPath):
           print "    ERROR: could not find subfolder %s to delete" % \
              targetFolderFSfullPath
        else:
           folder.deleteFolder(targetFolderFSfullPath)
           if not os.path.exists(targetFolderFSfullPath):
              print "    deleted folder: '%s'" % targetFolderFSfullPath
              if not os.path.exists(targetFolderMetadataFSfullPath):
                 testResult = 1
                 print "    verified that associated metadata file '%s' also deleted" % \
                    os.path.basename(targetFolderMetadataFSfullPath)
                    
    def test_getSiteMimetypes(self):
        """ Test if we can propeply build data needed for CSS mimetype icons
        """
        
        print
        print "test_test_getSiteMimetypes() ..."
        portal = self.portal
        folder = makeContent(portal, 'PloneLocalFolderNG', id='localfolder')
                
        folder.catalogFiles()
        
        plfng_utils = portal.plone_local_folder_tool
        icons = plfng_utils.getTypeIcons()
        normalized_names = map( lambda x: x["normalized_name"], icons)
                
        self.failUnless("pdf-png" in normalized_names)
        self.failUnless("doc-png" in normalized_names)
        self.failUnless("folder-icon-gif" in normalized_names)                                        

def test_suite():
    import unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(BaseTest))
    return suite

if __name__ == '__main__':
    framework(descriptions=1, verbosity=1)
