"""
    Functional tests for PloneLocalFolderNG basic views 
    
    Copyright 2006 Mikko Ohtamaa <mikko@redinnovation.com>
    
    Requires:
        - Plone 2.1

"""

__author__  = 'Mikko Ohtamaa <mikko@redinnovation.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os, sys
import time
from StringIO import StringIO
import urllib

if __name__ == '__main__':
    execfile(os.path.join(sys.path[0], 'framework.py'))

# Zope Imports
from Testing import ZopeTestCase

# Plone imports
from Products.Archetypes.atapi import *
from Products.Archetypes.tests.attestcase import default_user
from Products.Archetypes.tests.attestcase import HAS_PLONE21
from Products.Archetypes.tests.atsitetestcase import portal_owner
from Products.Archetypes.tests.atsitetestcase import ATFunctionalSiteTestCase
from Products.Archetypes.tests.utils import DummySessionDataManager
from Products.Archetypes.tests.utils import makeContent
from Products.CMFPlone.tests.PloneTestCase import PloneTestCase
from Products.CMFCore.utils import getToolByName
from Products.CMFCore.CMFCorePermissions import *

# Local imports
from Products.PloneLocalFolderNG import initialize
from Products.PloneLocalFolderNG.CatalogProxy import CatalogProxy
from Products.PloneLocalFolderNG.tests.test_plfng import SAMPLES_PATH
from PLFNGFunctionalTestCase import PLFNGFunctionalTestCase
from PLFNGFunctionalTestCase import print_sitemap

# Run __init__.py's initialize(context) for products used in this test case
# This registers types and other stuff
ZopeTestCase.installProduct('ZCatalog')
ZopeTestCase.installProduct('MimetypesRegistry')
ZopeTestCase.installProduct('Archetypes')
ZopeTestCase.installProduct('PloneLocalFolderNG')
ZopeTestCase.installProduct('PortalTransforms')

class TestView(PLFNGFunctionalTestCase):
    """ Functional tests for navigating inside PloneLocalFolderNG  
    """

    def afterSetUp(self):
        """ Do PloneLocalFolderNG navtree specific common test setup
        
        - Setup PloneLocalFolderNG instance into test environment        
        """
        PLFNGFunctionalTestCase.afterSetUp(self)

        plone_tool = self.portal.plone_utils    
                                                                   
    def test_walk_around(self):
        """ Test that we can propely move from a folder to another by clicking links
        
        """
        
        # test that subfolder is present first
        # print "fp:" + self.folder_path 
        response = self.publish(self.folder_path + "/localfolder/subfolder/plfng_view", self.basic_auth)
        self.failUnless(response.getStatus() == 200, "samples/subfolder not mounted")

        response = self.publish(self.folder_path + "/localfolder/subfolder/yetanothersubfolder/plfng_view", self.basic_auth)
        self.failUnless(response.getStatus() == 200, "samples/subfolder not mounted")                
        
        # go to local folder
        target = self.folder_path + "/localfolder"
        response = self.followLink(target)         
        
        target = self.folder_path + "/localfolder/subfolder"
        targetLink = "http://nohost" + target
        #print str(response.getBody())
        self.checkLink(response.getBody(), targetLink)        
        
        # go ot subfolder
        response = self.followLink(target)        
        # print "got to:" + response.getHeader("location")
        target = self.folder_path + "/localfolder/subfolder/yetanothersubfolder"
        targetLink = "http://nohost" + target
        self.checkLink(response.getBody(), targetLink)  
        response = self.followLink(target)
        

        response = self.followLink(self.folder_path + "/localfolder")
        target = self.folder_path + "/localfolder/sample.txt"                
        targetLink = "http://nohost" + target
        self.checkLink(response.getBody(), targetLink)          
        response = self.followLink(target)
        
        self.failUnless(response.getStatus() == 200, "Couldn't open sample.txt link")
           
    def followLink(self, address):
       """ Follow links and it's redirects until HTTP 200 answer or error
       
       @return final response after redirects
       """
       
       print "following to " + address
       
       # strip out http:// if publishing has added it
       if "http://" in address:
           address = address[len("http://nohost"):]
       
       response = self.publish(address, self.basic_auth)              
                 
       if(response.getStatus() == 302):
            loc = response.getHeader("location")
            #loc = loc[len("http://nohost"):]
            return self.followLink(loc)
       elif(response.getStatus() == 200):
           print "got 200 for " + address
           return response
       else:
           print "Response body:" + str(response.getBody())
           self.fail("Following link %s failed, HTTP response code %d" % (address, response.getStatus()))
                 
def test_suite():
    import unittest
    suite = unittest.TestSuite()
    suite.addTest(unittest.makeSuite(TestView))
    return suite

if __name__ == '__main__':
    # modify these settings to reduce output
    framework(descriptions=False, verbosity=False)