"""

    PloneLocalFolderNG utils
    
    Misc. utility functions
    
    Copyright 2006 Andreas Jung, Mike Garnsey, Mikko Ohtamaa

"""

__author__  = 'Mikko Ohtamaa <snaapy@gmail.com>'
__docformat__ = 'restructuredtext'

# Python imports
import os
from sys import platform
from stat import *

import shutil
import string
import re
from types import StringType, UnicodeType
import zipfile
import md5

try:
    import win32con, win32api
except ImportError:
    win32con = None
    win32api = None


# Zope imports
from AccessControl import ClassSecurityInfo
from DateTime import *
from ConfigParser import ConfigParser
from Acquisition import aq_base
import zLOG

# Plone imports
from Products.CMFCore.CMFCorePermissions import ModifyPortalContent

# Local imports
from FileProxy import FileProxy
from FileFilter import FileFilter

MD5_LENGTH = 32

# Note: Optional support for an external md5 generation system utility 
# has only been tested with the following utility:
#   * md5: md5deep v1.2 (http://md5deep.sourceforge.net/)

security = ClassSecurityInfo()

# --------------------------------------------------------------------
security.declareProtected(ModifyPortalContent, 'setMetadata')
def setMetadata(filename, section, option, value):

   metadataFileName = filename+".metadata"
   
   metadataFileParser = ConfigParser()
   metadataFileParser.read(metadataFileName)
   if not metadataFileParser.has_section(section):
      metadataFileParser.add_section(section)
   metadataFileParser.set(section,option,value)
   try:
      fp = open( metadataFileName, 'w')
      metadataFileParser.write(fp)
      fp.close()
      return 1
   except:
      return 0    

# --------------------------------------------------------------------
security.declarePublic('getMetadataElement')
def getMetadataElement(filename,section,option,REQUEST=None):
   metadataFileName = filename+".metadata"
   if os.path.exists(metadataFileName):
      try:
         metadataFileParser = ConfigParser()
         metadataFileParser.read(metadataFileName)
         lang=''
         if REQUEST:
            lang=REQUEST.get('LANGUAGE','')
         if (lang!='') and \
         (metadataFileParser.has_option(section, option+'_%s'%lang)):
            return metadataFileParser.get(section,option+'_%s'%lang)
         else:
            return metadataFileParser.get(section,option)
      except:
         #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
         # "getMetadataElement(%s,%s,%s)=None" % (section,option,filename))
         return None
   else:
      return None

# --------------------------------------------------------------------
security.declareProtected(ModifyPortalContent, 'getZipInfo')
def getZipInfo(filename):

   filename = fixDOSPathName(filename)
   
   z = zipfile.ZipFile(filename) 
   
   uncompressedSize = 0.0
   compressedSize = 0.0
   for zinfo in z.infolist():
      uncompressedSize = uncompressedSize + zinfo.file_size
      compressedSize = compressedSize + zinfo.compress_size
   
   if uncompressedSize > 1.0:
      compressionRatio = 100.0 * (1.0 - compressedSize/uncompressedSize)
   else:
      compressionRatio = 0.0
   
   return len(z.infolist()), uncompressedSize
   
   # here is the code to mimic 'unzip -Zt' output
   # result = "%d files, %d bytes uncompressed, %d bytes compressed: %.1f%%"
   #    % (len(z.filelist), uncompressedSize, compressedSize, compressionRatio)
   
   # this is the code from PLFNG 0.5 that used an external system
   # call to the UnZip v5.50 utility.  Here is its output format: 
   #  21 files, 50550 bytes uncompressed, 12409 bytes compressed:  75.5%
   # -------------------------------------------------------------
   #SYSCALL_ZIPINFO         = "c:\\plone-utils\\unzip -Zt"
   #syscall = "%s %s" % (SYSCALL_ZIPINFO,syscall_filename)
   #process_fp = os.popen(syscall)
   #result = process_fp.read()
   # -------------------------------------------------------------   


# --------------------------------------------------------------------
security.declareProtected(ModifyPortalContent, 'setZipInfoMetadata')
def setZipInfoMetadata(filename):

   if zipfile.is_zipfile(filename): 
      num_files, unpacked_size = getZipInfo(filename)
  
      setMetadata(filename,
                  section="ARCHIVEINFO",
                  option="num_files",
                  value=num_files)
      setMetadata(filename,
                  section="ARCHIVEINFO",
                  option="unpacked_size",
                  value=unpacked_size)
      return 1

   else:
      return 0

security.declareProtected(ModifyPortalContent, 'upzipFile') 
def upzipFile(PLFNG_obj, plfngRelativeDir, FSfilename):
   """ Unzip the content of uploaded zip file
     
     this method does the same thing as the following system call:
     SYSCALL_UNZIP = "c:\\plone-utils\\unzip -o -d"
     syscall = "%s %s %s" % (SYSCALL_UNZIP,todir,syscall_filename)
     result = os.system(syscall) """

   zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "upzipFile(%s,%s,%s)" % \
   (PLFNG_obj, plfngRelativeDir, FSfilename))
   
   overwriteExistingFiles = getattr(PLFNG_obj, 'overwrite_existing_files', 0)

   filesNotExtracted = 0
   filesExtracted = 0

   FSfilename = fixDOSPathName(FSfilename)
   #todir = os.path.dirname(FSfilename)
   
   todir = os.path.join(PLFNG_obj._getFSBasePath(),plfngRelativeDir.replace('/', os.sep) )

   zLOG.LOG('PloneLocalFolderNG', zLOG.INFO, "todir=%s" % todir)
   if zipfile.is_zipfile(FSfilename): 

      #print "upzipFile(): FSfilename=%s, todir=%s" % (FSfilename,todir)
      
      z = zipfile.ZipFile(FSfilename)

      # create the extract directory structure before extracting files to it
      for zipitem_filename in z.namelist():
         if zipitem_filename.endswith('/'):
            newDirectoryName = os.path.join(todir, zipitem_filename)
            if not os.path.exists(newDirectoryName):
                os.makedirs(newDirectoryName)

      # extract the files from the zipfile
      FStmpfile = os.path.join(todir, 'unpack.plfngtmp')
      #for zipitem_filename in z.namelist():
      for currentZipInfoItem in z.infolist():
         zipitem_filename = currentZipInfoItem.filename
         time_str = "%d-%02d-%02d %02d:%02d:%02d" % currentZipInfoItem.date_time
         zipitem_mtime = DateTime(time_str).timeTime()
         if not zipitem_filename.endswith('/'):
            fname = os.path.basename(zipitem_filename)
            checkValidIdResult = checkValidId(os.path.basename(fname))
            if checkValidIdResult != 1:
               zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                 "upzipFile() :: checkValidId(%s) failed:: %s" \
                 % (zipitem_filename,checkValidIdResult) ) 
            else:
               tofile = os.path.join(todir, zipitem_filename)
               oldRevisionNumber = 0
               newRevisionNumber = 1
               # if file already exists and its ok to overwrite it,
               # try to delete it via deleteFile() method --which will
               # take care of read-only attribute checking, and file backup

               skipThisFile = 0
               if os.path.exists(tofile):
                  if overwriteExistingFiles:
                     #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                     #  "upzipFile() :: attempting to delete file %s " % tofile ) 

                     PLFNG_obj.deleteFile(tofile,
                                          REQUEST=None,
                                          RESPONSE=None)
                     if os.path.exists(tofile):
                        skipThisFile = 1
                        zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                          "upzipFile() :: failed to delete file %s " % tofile ) 
                  else:
                     skipThisFile = 1

               if skipThisFile:
                  filesNotExtracted = filesNotExtracted + 1
                  zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                   "upzipFile() :: extract failure for file=%s" % tofile )
               else:
                  # extract the file
                  try:
                     f = open(FStmpfile, 'wb')
                     f.write(z.read(zipitem_filename))
                     f.close()
                     
                     # Set modification time of extracted file according its zipinfo
                     os.utime(FStmpfile, (zipitem_mtime, zipitem_mtime))
                     
                     if plfngRelativeDir:
                        destPLFNGrelativePath = \
                        plfngRelativeDir+"/"+os.path.dirname(zipitem_filename)
                     else:
                        destPLFNGrelativePath = os.path.dirname(zipitem_filename)
                        
                     destPLFNGrelativePath = destPLFNGrelativePath.replace('/', os.sep)
                     #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                     #"upzipFile() :: addFile() sourceFSfullPath=%s,destPLFNGrelativePath=%s" \
                     # % (FStmpfile,destPLFNGrelativePath) )
                     if PLFNG_obj.addFile(sourceFSfullPath=FStmpfile,
                                        destPLFNGrelativePath=destPLFNGrelativePath,
                                        destBasename=os.path.basename(tofile),
                                        moveFile=1,
                                        inputMD5=None):
                        filesExtracted = filesExtracted + 1
                     else:
                        filesNotExtracted = filesNotExtracted + 1
                        zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                        "upzipFile() :: extract failure for file=%s" % tofile )   
                  except:
                     filesNotExtracted = filesNotExtracted + 1
   

      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      #"upzipFile() finished unzip'ing %s" % FSfilename)
      return filesExtracted,filesNotExtracted

   else:
      zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
               "upzipFile() ABORTED: %s not a valid zip file." % FSfilename )
      return 0,0    

# --------------------------------------------------------------------
security.declareProtected(ModifyPortalContent, 'zipFiles') 
def zipFiles(PLFNG_obj, destFilePLFNGRelRelativePathName, filesList):

   # NOTE!!!!! do not use this yet.....it is unfinished!
   
   FSfilename = os.path.join(PLFNG_obj.getFileRealPath(),
                             destFilePLFNGRelRelativePathName.replace('/', os.sep) )
   
   if 1:

      z = zipfile.ZipFile(FSfilename,"w")
      for filename in filesList:
          z.write(filename,os.path.basename(filename))
      
      z.close()    



# --------------------------------------------------------------------
security.declareProtected(ModifyPortalContent, 'generate_md5')
def generate_md5(filename,syscallcommand='none'):

   filename = fixDOSPathName(filename)
   md5_hexvalue = ''

   if syscallcommand == 'none':
      m = md5.new()
      blocksize = 2<<16
      try:
         fobj = open(filename, 'rb')
         while True:
             chunk = fobj.read(blocksize)
             if not chunk:
                 break
             m.update(chunk)
         md5_hexvalue = m.hexdigest()
         fobj.close()
      except:
         md5_hexvalue = ''    

   else:
      fullsyscall = "%s %s" % (syscallcommand,filename)
      process_fp = os.popen(fullsyscall)
      md5_hexvalue = process_fp.read()[:MD5_LENGTH]

   if len(md5_hexvalue) == MD5_LENGTH:
      return md5_hexvalue
   else:
      # failure should throw an exception, but for now, return 
      # easily identified bogus checksum 
      return "99999999999999999999999999999999"

# --------------------------------------------------------------------
def determine_bytes_usage(startObject):
    """ 
        attempt to determine the total size in bytes of all subobjects of 
        object.  Note: this only gathers sizes of objects that have the 
        'get_size' attribute.
    """
    # not used now, but might come in handy at some point
    exclude_quota_objects = []
    
    obj_sizes = [ obj.get_size() for id, \
       obj in startObject.ZopeFind( startObject, search_sub=1 ) \
       if hasattr( aq_base( obj ), 'get_size') and \
       getattr( obj, 'meta_type', None ) not in exclude_quota_objects ]
    if obj_sizes:
        objects_size = int( reduce( lambda x,y: x+y, obj_sizes ) )
    else:
        objects_size = 0

    return objects_size

# --------------------------------------------------------------------
def fixDOSPathName(srcFileName):
    fixedName=string.join(string.split(srcFileName,'\\'),'\\\\')
    fixedName=string.join(string.split(fixedName,'\t'),'\\\\t')
    fixedName=string.join(string.split(fixedName,'\n'),'\\\\n')
    return fixedName

# --------------------------------------------------------------------
def catalogFSContent(FSfullPath, FSbasePath, filetypePhrasesSkipList, catalogTool, 
                     mimetypesTool, uidBase, view_roles, effective, expires,
                     meta_type, catalog_contents, catalog_paths):
   """ Catalog file system content 
     
   Makes mounted local folder contents searchable and adds navtree support.
   This might be an expensive operation if there is a lot of contents to be
   cataloged. 
     
   @param FSfullPath full system path to the file to be cataloged
   @param FSbasePath full system path to the PloneLocalFolderNG mount point
   @param filetypePhrasesSkipList Matcing filenames to these patterns are not cataloged
   @param catalogTool portal_catalog tool
   @param mimetypesTypes portal_mimetypes tool
   @param uidBase Portal URL base for the contents
   @param viewRoles
   @param effective
   @param expires
   @param meta_type
   @param catalog_contents File contents is indexed as text and made to be available in site search
   @param catalog_paths Filenames and paths are put ExtendedPathIndex. Needed for navtree support.
     
   """
   
   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
   #   "catalogFSContent(%s,%s,%s,%s,%s,%s,%s,%s,%s)" \
   #   % (FSfullPath, filetypePhrasesSkipList, catalogTool, mimetypesTool, \
   #    uidBase, view_roles, effective, expires, meta_type) )

   # uidBase = str('/' + portalId + '/'+ \
   # this_portal.getRelativeContentURL(self) + '/')
   
   filesCataloged = 0

   fileSuffixesSkipList=['.metadata', '.plfngtmp']
   
   # 1st, get the filtered lists of files and folders to catalog
   filteredFileList = []
   filteredFolderList = []
   
   if os.path.isdir( FSfullPath ):
      FSfullPathFolderName = FSfullPath
      
      filteredFileList, filteredSubfolderItems = \
         getFilteredFSItems(FSfullPath=FSfullPathFolderName, 
                            skipInvalidIds=1, 
                            mimetypesTool=mimetypesTool, 
                            filetypePhrasesSkipList=filetypePhrasesSkipList, 
                            filePrefixesSkipList=[], 
                            fileSuffixesSkipList=fileSuffixesSkipList, 
                            fileNamesSkipList=[], 
                            folderPrefixesSkipList=[], 
                            folderSuffixesSkipList=[], 
                            folderNamesSkipList=[] )
   else:
      FSfullPathFolderName = os.path.basename( FSfullPath )
      FSfullPathFileName = FSfullPath
      fileName = os.path.basename( FSfullPathFileName )
      
      skipThisItem = 0
      checkValidIdResult = checkValidId( fileName )
      if checkValidIdResult != 1: 
         skipThisItem = 1
         zLOG.LOG( 'PloneLocalFolderNG', zLOG.INFO , \
         "catalogFSContent() :: checkValidId(%s) failed:: %s" \
         % ( FSfullPathFileName, checkValidIdResult ) )
      for suffix in fileSuffixesSkipList:
         if fileName.endswith( suffix ): 
            skipThisItem = 1
            break

      if skipThisItem != 1:
         filteredFileList.append( fileName )
   
   # instantiate a barebones FileProxy instance
   dummyFileProxy = FileProxy( id="dummy", 
                              filepath=FSfullPath, 
                              fullname="dummy", 
                              properties=None )
   
   dummyFileProxy._View_Permission = view_roles
   dummyFileProxy.meta_type = meta_type
   dummyFileProxy.effective = effective
   dummyFileProxy.expires = expires
   
      
   # catalog all of the files in the filtered file list
   for fileItem in filteredFileList:
      FSfullPathFileName = os.path.join( FSfullPathFolderName, fileItem )

      uid = str(uidBase +  os.path.basename(FSfullPathFileName))
                 
      # relative file system path from mount point to this item 
      relativePath = makeRelativePath(FSfullPathFileName, FSbasePath)
            
      mi = mimetypesTool.classify(data=None, filename=fileItem.lower())
      try:
         item_mime_type = mi.normalized()
      except:
         item_mime_type = mimetypesTool.defaultMimetype
 
      try:
          icon_path = mi.icon_path
      except:
          icon_path = 'unknown.png'

      dummyFileProxy = FileProxy(id=str(fileItem),
                                 url = FSfullPathName,
                                 encoding = None,
                                 path=makePortalPath(uidBase, relativePath),
                                 icon_path = icon_path,
                                 mime_type = item_mime_type,
                                 
                                 )

      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      #"catalogContents() :: file=%s" % FSfullPathFileName )
      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      #"catalogContents() :: mimetype=%s" % dummyFileProxy.mime_type )
      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      #"catalogContents() :: catalog uid=%s" % uid )
                 
      catalogTool.catalog_object(dummyFileProxy, uid)
                   
      filesCataloged = filesCataloged + 1
   
   # catalog all the contents of the subfolders in the filtered subfolders list
   for subfolder in filteredSubfolderItems:
      subfolderFullName = os.path.join( FSfullPathFolderName, subfolder )
      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      #"catalogFSContent() :: subdir=%s" % subfolderFullName )
      newUidBase = uidBase + subfolder + '/'
      subfolderfilesCataloged = \
         catalogFSContent( subfolderFullName, FSbasePath, filetypePhrasesSkipList, 
                          catalogTool, mimetypesTool, newUidBase, view_roles, 
                          effective, expires, meta_type, catalog_contents, catalog_paths )
      filesCataloged = filesCataloged + subfolderfilesCataloged

   return filesCataloged


# --------------------------------------------------------------------
def getFilteredFSItems(FSfullPath, skipInvalidIds, mimetypesTool, 
                       filetypePhrasesSkipList, filePrefixesSkipList,
                       fileSuffixesSkipList, fileNamesSkipList,
                       folderPrefixesSkipList, folderSuffixesSkipList,
                       folderNamesSkipList):
   
   # this method returns the filteredFileList & filteredFolderList children of 
   # the specified filesystem (FS) path
   
   filteredFileList = []
   filteredFolderList = []
   
   if not os.path.exists(FSfullPath):
      # raise exception here?
      zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      "getFilteredFSItems() :: FSfullPath not found (%s)" % FSfullPath )
      
   else:   
      try:
         rawItemList = os.listdir(FSfullPath)
      except:
         zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
         "getFilteredFSItems() :: error reading folder (%s) contents" \
         % FSfullPath )
         return filteredFileList, filteredFolderList

      for item in rawItemList:
         if skipInvalidIds:
            checkValidIdResult = checkValidId(item)
            if checkValidIdResult != 1: 
               #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
               #"getFilteredFSItems() :: checkValidId(%s) failed:: %s" \
               #% (item,checkValidIdResult) )
               continue
      
         FSfullPathItemName = os.path.join(FSfullPath, item)
         skipThisItem = 0
         
         if os.path.isdir(FSfullPathItemName):
             
             for prefix in folderPrefixesSkipList:
                if item.startswith(prefix): 
                   skipThisItem = 1
                   break
             
             for suffix in folderSuffixesSkipList:
                if item.endswith(suffix): 
                   skipThisItem = 1
                   break
             
             for completeName in folderNamesSkipList:
                if item == completeName: 
                   skipThisItem = 1
                   break
             
             if not skipThisItem: 
                filteredFolderList.append(item)
         
         else:
             
             if mimetypesTool:
                mi = mimetypesTool.classify(data=None, filename=item.lower())
                try:
                   item_mime_type = mi.normalized()
                except:
                   item_mime_type = mimetypesTool.defaultMimetype
                   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                   # "warning: no filename mime type for %s" % filename)
             
                for filetypePhrase in filetypePhrasesSkipList:
                   if item_mime_type.find(filetypePhrase) >= 0:
                      skipThisItem=1
                      break
             
             for prefix in filePrefixesSkipList:
                if item.startswith(prefix): 
                   skipThisItem = 1
                   break
             
             for suffix in fileSuffixesSkipList:
                if item.endswith(suffix): 
                   skipThisItem = 1
                   break
             
             for completeName in fileNamesSkipList:
                if item == completeName: 
                   skipThisItem = 1
                   break
             
             if not skipThisItem: 
                filteredFileList.append(item)
     
   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
   #"filteredFolderList= %s" % filteredFolderList)
   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , "filteredFileList= %s" \
   #% filteredFileList)
   
   return filteredFileList, filteredFolderList
   
   
# --------------------------------------------------------------------
def getFilteredOutFSItems(FSfullPath, PLFNGrelPath, skipInvalidIds,
                          mimetypesTool, filetypePhrasesSkipList, 
                          filePrefixesSkipList, fileSuffixesSkipList,
                          fileNamesSkipList,folderPrefixesSkipList,
                          folderSuffixesSkipList, folderNamesSkipList):
   
   # this method is the inverse of the getFilteredFSItems().  It returns
   # the file and folder children of the specified filesystem (FS) path
   # that are ***NOT*** returned by the getFilteredFSItems() method
   
   if PLFNGrelPath != '':
      PLFNGrelPath = PLFNGrelPath.replace('\\', '/') + '/'

   illegalFilesList = []
   illegalFoldersList = []
   filteredOutFileList = []
   filteredOutFolderList = []
   
   if not os.path.exists(FSfullPath):

      # raise exception here?
      zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
      "getFilteredOutFSItems() :: FSfullPath not found (%s)" % FSfullPath )
      
   else:   
      try:
         rawItemList = os.listdir(FSfullPath)
      except:
         zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
         "getFilteredOutFSItems() :: error reading folder (%s) contents" \
         % FSfullPath )
         return filteredFileList, filteredFolderList

      for item in rawItemList:
         FSfullPathItemName = os.path.join(FSfullPath, item)
         skipThisItem = 0
         
         if os.path.isdir(FSfullPathItemName):
             checkValidIdResult = checkValidId(item)
             if skipInvalidIds and checkValidIdResult != 1:
                illegalFoldersList.append(PLFNGrelPath+item)
                #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                #"getFilteredOutFSItems() :: checkValidId(%s) failed:: %s" \
                #% (item,checkValidIdResult) )
             else:
                for prefix in folderPrefixesSkipList:
                   if item.startswith(prefix): 
                      skipThisItem = 1
                      break
                
                for suffix in folderSuffixesSkipList:
                   if item.endswith(suffix): 
                      skipThisItem = 1
                      break
                
                for completeName in folderNamesSkipList:
                   if item == completeName: 
                      skipThisItem = 1
                      break
                
                if skipThisItem: 
                   filteredOutFolderList.append(PLFNGrelPath+item)
         
         else:
             checkValidIdResult = checkValidId(item)
             if skipInvalidIds and checkValidIdResult != 1:
                illegalFilesList.append(PLFNGrelPath+item)
                #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                #"getFilteredOutFSItems() :: checkValidId(%s) failed:: %s" \
                #% (item,checkValidIdResult) )
             
             else:
                if mimetypesTool:
                   mi = mimetypesTool.classify(data=None,filename=item.lower())
                   try:
                      item_mime_type = mi.normalized()
                   except:
                      item_mime_type = mimetypesTool.defaultMimetype
                      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
                      # "warning: no filename mime_type for %s" % filename)
                
                   for filetypePhrase in filetypePhrasesSkipList:
                      if item_mime_type.find(filetypePhrase) >= 0:
                         skipThisItem=1
                         break
                
                for prefix in filePrefixesSkipList:
                   if item.startswith(prefix): 
                      skipThisItem = 1
                      break
                
                for suffix in fileSuffixesSkipList:
                   if item.endswith(suffix): 
                      skipThisItem = 1
                      break
                
                for completeName in fileNamesSkipList:
                   if item == completeName: 
                      skipThisItem = 1
                      break
             
                if skipThisItem: 
                   filteredOutFileList.append(PLFNGrelPath+item)
     
   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
   #"filteredOutFolderList= %s" % filteredOutFolderList)
   #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , \
   #"filteredOutFileList= %s" % filteredOutFileList)
   
   return illegalFilesList, illegalFoldersList,\
          filteredOutFileList, filteredOutFolderList


# --------------------------------------------------------------------
def _makeFileDeletable(FSfullPath):
    # this method provides a crude way to override a file's OS permission 
    # settings intended to protect the file from being deleted.  
    # OS differences complicate detection of and override of file
    # permissions, as the following notes describe:
    #
    # in Unix, write permission is not the same thing as delete permission
    # (which requires write permission on the directory, and ownership of
    # the file if the directory has the sticky bit set).
    #
    # in NTFS, to prevent a file from being deleted, you must ensure that
    # the user is not granted the Delete Subfolders and Files special
    # permission on the parent folder (MyFolder), explicitly or through
    # group membership. To do this, use the Effective Permissions tab to
    # view the folder's special permissions that are granted to the user.
    # If Delete Subfolders and Files is selected, the user can delete all
    # files within the folder.
    if platform == 'win32':
        try:
           if win32api and win32con:
              win32api.SetFileAttributes(fileToDelete,
                                         win32con.FILE_ATTRIBUTE_NORMAL)
              okToDeleteFile = 1
        except:
           pass
    else:
        try:
           if hasattr(os, 'chmod'):
              os.chmod(fileToDelete, 0600)
              okToDeleteFile = 1
        except OSError:
           pass
        
# --------------------------------------------------------------------
def _isFileDeletable(FSfullPath):
    # this method provides a crude way to determine if a file's OS 
    # permission settings allow for the file to be deleted. See the 
    # notes in the _makeFileDeletable() method for details on the
    # complexity of determining this information.
    
    fileHasDeletePermission = 1
    
    st = os.stat(FSfullPath)
    mode = st[ST_MODE]	
    if mode & S_IWRITE:
       fileHasDeletePermission = 1
    else:
       fileHasDeletePermission = 0

    #msg = '_isFileDeletable() :: fileHasDeletePermission=%d' \
    # %fileHasDeletePermission
    #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)        
   
    return fileHasDeletePermission
    
    
def makeRelativePath(fullpath, basepath):
    """ Returns fullPath relative to basePath
    
    E.g. if fullpath is /usr/src/moo and basePath is /usr
    return src/moo    
    
    @param fullpath Full path to a file as a string
    @param basebath Base as a string (must be trimmed)
    """
    rel_dir = fullpath.replace(basepath, '')
    rel_dir = rel_dir.replace('\\','/')
    if rel_dir.startswith('/'): rel_dir = rel_dir[1:]
    return rel_dir
    
def makePortalPath(mountPoint, relativePath):
    path = mountPoint.split("/") + relativePath.split("/")
    # remove empty entries (caused by trailing and prefixing slash)
    path = filter( lambda x: x != "", path)
    
    # Aaargh
    # The world almost lost poor Moo before 
    # he figured out the mysterious first empty element
    # in portal path. Why it's there? Who knows...
    # but I recommend *not* to try to figure out it.
    # Rather, making a meal from radioactive toxins 
    # sounds like a better idea
    return ("",) + tuple(path)
    
def getFolderProperties(folder_path):
    """ Get unfiltered folder properties
    
    Recursively count files and their sizes   
    """
    filteredFileList = []
    filteredFolderList = []
    bytesInFolder = 0
    folderCount = 0
    fileCount = 0
   
    file_filter = FileFilter(
                         skipInvalidIds=1,
                         mimetypesTool=None,
                         filetypePhrasesSkipList=[],
                         filePrefixesSkipList=[],
                         fileSuffixesSkipList=['.metadata','.plfngtmp'],
                         fileNamesSkipList=[],
                         folderPrefixesSkipList=[],
                         folderSuffixesSkipList=[],
                         folderNamesSkipList=[])
                         
    
    filteredFileList, filteredFolderList = file_filter.listFolder(folder_path)

    fileCount = fileCount + len(filteredFileList)
    folderCount = folderCount + len(filteredFolderList)

    for f in filteredFileList:
        FSfullPathFileName = os.path.join(folder_path,f)
        try: file_size = os.stat(folder_path)[6]
        except: file_size = 0
        bytesInFolder = bytesInFolder + file_size

    for subfolder in filteredFolderList:
        subfolderFullName = os.path.join(folder_path, subfolder)

      #msg = '_getFolderProperties() :: calling _getFolderProperties('+subfolderFullName+')'
      #zLOG.LOG('PloneLocalFolderNG', zLOG.INFO , msg)

        subfolder_props = getFolderProperties(subfolderFullName)
        bytesInFolder = bytesInFolder + subfolder_props.get('size',0)
        folderCount = folderCount + subfolder_props.get('folders',0)
        fileCount = fileCount + subfolder_props.get('files',0)

    folderProps = { }
    folderProps['size'] = bytesInFolder
    folderProps['files'] = fileCount
    folderProps['folders'] = folderCount
    return folderProps
   
   
    def __setattr__(self, item, value):
        if "item" == "plfng_view":
            raise "Shit"
        else:
            BaseContent.___setatrr__(self, item, value)    

def makeValidPath(path):

    if path.startswith('/'): path = path[1:]
    if path.endswith('/'): path=path[0:len(path)-1]
    pathparts = path.split('/')
    
    fixedPathParts = []
    for part in pathparts:
        #if not id: no need to fix this
        if bad_id(part) is not None:
            part = part #need to implement this!!! (regex that replaces all bad chars with '~'?)
        #if id in ('.', '..'):  #no fix for this!
        if part.startswith('_'):
            part = '~~' + part
        if part.startswith('aq_'):
            part = '~~' + part
        if part.endswith('__'):
            part = part + '~~'
        if part == 'REQUEST':
            part = '~~' + part
        #if '/' in id: #no fix for this!
        fixedPathParts.append(part)
    
    fixedPath = '/'.join(fixedPathParts)
    return fixedPath
               